<!DOCTYPE html>
<html>

<head>
  {% block head %}
    <meta charset="utf-8" />
    <title>Cognitive Study</title>
    <link rel="Favicon" href="" />
    <link rel="stylesheet" href="/static/css/bootstrap.min.css" type="text/css" />
    <link rel="stylesheet" href="/static/css/style.css" type="text/css" />
    <link rel="icon" href="/static/favicon.ico" />
  {% endblock %}
</head>

<body>
  {% block body %}
  {% endblock %}
</body>

</html>
